/*  Copyright 2008 Peter Mora
 *
 *  This file is part of Vakarcs.
 *
 *  Vakarcs is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Vakarcs is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Vakarcs.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef _OBJECT_H
#define _OBJECT_H

#include <stdlib.h>
#include <iostream>
#include <vector>

#define COOLDOWN 3
#define ABILITY_SHOOT 1
#define ABILITY_SHOOT_PART 60

using namespace std;

class Server;
class Ability;
class PlayerInfo;


class Object{
	int		m_id;
	string		m_name;

	int		m_red;
	int		m_blue;
	int		m_green;
	int		m_white;
	int		m_black;

	int		m_owner;
	string		m_step;
	int		m_stepx;
	int		m_stepy;
	int		m_posx, m_posy;

	vector<int>	m_wall_change;
	int		m_wall_change_to;

	int		m_active;	// 0 : no
	int		m_visible;	// 1 : yes
					// n<0 : no, but it will be yes in n rounds
					// n>1 : yes, but it will be no in n-1 rounds
					//
	int		m_blocked;

	int		m_cooldown[COOLDOWN];

	void		destroy(bool* redraw,int x, int y,Server* server,PlayerInfo* own_playerinfo, string own_before, string own_after, string target_before, string target_after, int damage = 1, Object* ob = NULL);
	void		step_parameter(int* which);
	void		block(Server* server, int range);
public:
	Object();
	Object(Object* ob);
	Object(string st);
	string		generate_string(int mirror = 0, int maxx = 0, int maxy = 0);
	~Object();
//	string		action(int part, string type, int x, int y, Server* server);
	string		action(int part, Server* server,bool* redraw, PlayerInfo* own_playerinfo, vector<Ability*>* ab);
	string		ability(int ab_num, int part, Server* server,bool* redraw, PlayerInfo* own_playerinfo,  vector<Ability*>* ab);
	int		properties(string type, int red = -1, int blue = -1, int white = -1, int green = -1, int black = -1);
	int		id(){return m_id;};
	int		owner(){return m_owner;};

	void		step(string step, int x, int y) { m_step = step; m_stepx = x; m_stepy = y;};

	int		x(){return m_posx;}
	int		y(){return m_posy;}


	string		name(){return m_name;}
	int		red(){return m_red;}
	int		blue(){return m_blue;}
	int		white(){return m_white;}
	int		green(){return m_green;}
	int		black(){return m_black;}
	int		active(){return m_active;}
	void		active(int a){m_active=a;}
	int		visible(){return m_visible;}
	int		blocked(){return m_blocked;}
	void		blocked(int b){m_blocked = b;}
	int*		cooldown(){return m_cooldown;}


	int		future_x[10];
	int		future_y[10];
	string		future_step[10];
//	int		future_posx[10];
//	int		future_posy[10];
	bool		has_ability(string st, vector<Ability*>* ab);
	int		ability_parameter(string st, vector<Ability*>* ab);
	void		bomb_on_death(int range, Server* server, bool* redraw);
};

#endif
