/*  Copyright 2008 Peter Mora
 *
 *  This file is part of Vakarcs.
 *
 *  Vakarcs is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Vakarcs is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Vakarcs.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef _SERVER_H
#define _SERVER_H

#include <stdlib.h>
#include <iostream>
#include <vector>
#include "object.h"

using namespace std;

#define VERSION "0.61"

#define MAP_WALL -1
#define MAP_EMPTY -2
#define MAP_DESTROYED -3
#define MAP_BASE 1000 // for player i it's 1000+i

#define MAPS_BLIND   0
#define MAPS_VISIBLE 1
#define MAPS_MARKED 10000

#define MAX_VAKARCS 1
// #define FIRST_UNDIRECTED_ABILITY 5

#ifdef __WIN32__
#include "windows.h"
static void   usleep(int n)
{
       Sleep(n/1000);
}
#endif

struct Round{
	vector<Object*>		objects;
//	vector<std::string>	messages;
	int*			map;
	int*			visible_maps_for_players;
	int*			score;
};


struct PlayerInfo{
	int		round;
	vector<string>	strings;
	int*		map_for_player;
	vector<Object*>	objects;
	int*		score;
};

struct Startscript{
	int		round;
	string		object;
};

struct Ability{
	int	red;
	int	green;
	int	blue;
	int 	white;
	vector<string>	ability;
	vector<int>	id;
	vector<int>	parameter;
	vector<int>	cooldown;
	int		number[10];
};

class Server{
	bool	m_game_over;
	string	m_abilities_types;
	int	m_number_of_players;
	bool*	m_active_players;
	int*	m_bases;
	int*	m_mirror;

	int	m_last_id;
	int	m_vak_number;
	int	m_vak_number_down;
	
	vector<string>*		m_names;

	vector<Round*>		m_round;
	vector<PlayerInfo*>*	m_player_info;
	vector<PlayerInfo*>	m_next_player_info;
	int*			m_first_info_player_not_has;

	int			m_copy_round_from;

	int*			m_punishment_from_timeback;

	vector<string>		m_outgoing_messages;
	vector<string>		m_incoming_messages;

	vector<Startscript*>	m_startscript;

	int	m_round_time;
	int	m_to_win;

	vector<Ability*>	m_abilities;

	int	m_maxx;
	int	m_maxy;

	void	set_up_step(Round* r, string step, int owner); 
	void	generate_messages();
	int	mirrorx(int x, int y, int p)
	{ 
		if(m_mirror[p] == 0 || m_mirror[p] == 2) return x;
		if(m_mirror[p] == 1 || m_mirror[p] == 3) return m_maxx - x -1;
		if(m_mirror[p] == 4 || m_mirror[p] == 6) return y;
		if(m_mirror[p] == 5 || m_mirror[p] == 7) return m_maxy - y -1;
	}
	int	mirrory(int x, int y, int p)
	{ 
		if(m_mirror[p] == 0 || m_mirror[p] == 1) return y;
		if(m_mirror[p] == 2 || m_mirror[p] == 3) return m_maxy - y -1;
		if(m_mirror[p] == 4 || m_mirror[p] == 5) return x;
		if(m_mirror[p] == 6 || m_mirror[p] == 7) return m_maxx - x -1;
	}


public:
	Server();
	~Server();
	string init(int n);
	bool is_gameover() { return m_game_over; }
	int	mirror(int p){return m_mirror[p];};
	std::string send(int playerid);
	void receive(int playerid,std::string str);
	void make_rounds();
	void check_game_over();
	void make_step();
	Round*		actual_round(){ return m_round[m_round.size()-1]; }
	int	maxx () { return m_maxx;}
	int	maxy () { return m_maxy;}
	int	map(int x, int y) { return m_round[m_round.size()-1]->map[y*m_maxx+x];};
	int	map(int i) { return m_round[m_round.size()-1]->map[i];};
	int*	own_visible_map(int player, int x, int y) { return &(m_round[m_round.size()-1]->visible_maps_for_players[m_maxx*m_maxy*player+y*m_maxx+x]);};

	void	generate_maps(int playerid);
	void	score(int player, int n);
	vector<PlayerInfo*>*	next_player_info(){return &m_next_player_info;};
	vector<Ability*>*	abilities(){ return &m_abilities;}
	int	number_of_players(){return m_number_of_players;}

	int	timeback(int number_of_rounds, int punishment, int for_who);
	void	leaver(int n);
	bool*	active_players(){return m_active_players;}
};

static vector<std::string>*	tokenize(std::string str,char ch,bool last = false)
{
	std::string q;
	vector<std::string>*	r = new vector<std::string>;
	for (int i = 0; i < str.size(); i++)
	{
		if (str[i] == ch)
		{
			r->push_back(q);
			q = "";
		}
		else
			q = q+str[i];
	}
	if (last)
		r->push_back(q);
	return r;
};

static string	itostr(int i)
{
	char s[50];
	sprintf(s,"%d",i);
	return string(s);
};

static int	MIN(int x, int y)
{
	if (x < y)
		return x;
	return y;
};

static int	MAX(int x, int y)
{
	if (x > y)
		return x;
	return y;
};

static void	debug(string st)
{
	cout << st << "\n";
	cout.flush();
	char ch;
	scanf("%c",ch);
}

#endif
