/*  Copyright 2008 Peter Mora
 *
 *  This file is part of Vakarcs.
 *
 *  Vakarcs is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Vakarcs is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Vakarcs.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "ui.h"
#include "server.h"
#include "math.h"

Ui::Ui()
{
	m_chosen_object = 0;
	m_current_round = 0;
//	m_last_round = 0;
	m_current_info = 0;
	m_current_info_string = 0;
	m_mode = 0;
	m_game_over = false;
	m_wall_type = 0;
	setlocale(LC_CTYPE,"hu_HU.UTF-8");
}

Ui::~Ui()
{
	for(int i = 0; i < m_abilities.size(); i++)
		delete m_abilities[i];
	for(int i = 0; i < m_player_info.size(); i++)
	{
		delete[] m_player_info[i]->map_for_player;
		for (int j = 0; j < m_player_info[i]->objects.size(); j++)
			delete m_player_info[i]->objects[j];
		delete[] m_player_info[i]->score;
		delete m_player_info[i];
	}
	for(int i = 0; i < m_translate.size(); i++)
		delete m_translate[i];
}

void	Ui::set_abilities(string str)
{
	vector<string>* t = tokenize(str,'^');
	m_abilities_types = (*t)[0];
	for (int i = 1 ; i < t->size(); i++)
	{
		vector<string>* s = tokenize((*t)[i],' ');
		Ability* ab = new Ability;
		ab->red = atol((*s)[0].c_str());
		ab->green = atol((*s)[1].c_str());
		ab->blue = atol((*s)[2].c_str());
		ab->white = atol((*s)[3].c_str());
		for (int q = 0; q < 10; q++)
			ab->number[q] = -1;
		for (int j = 4; j < s->size(); j+=3)
		{
			int w = (*s)[j][0] - 48;
			if (w >= 0 && w < 10)
			{
				ab->number[w] = ab->ability.size();
				ab->id.push_back(w);
			}
			else
				ab->id.push_back(0);
			ab->ability.push_back((*s)[j].substr(1,(*s)[j].size()-1));
			ab->parameter.push_back(atol((*s)[j+1].c_str()));
			ab->cooldown.push_back(atol((*s)[j+2].c_str()));
		}
		m_abilities.push_back(ab);
		delete s;
	}
	delete t;
}

void	Ui::info_print(const char* st, int fore, int back)
{
	int q = strlen(st);
	move(MAP_OFFSET+m_maxy+m_info_print_line,m_info_print_pos);
	color_print(MAP_OFFSET+m_maxy+m_info_print_line,m_info_print_pos,st,fore,back);
	int x,y;
	getyx(stdscr,y,x);
	m_info_print_pos += x-m_info_print_pos;
//	m_info_print_pos += q;
}

void	Ui::color_print(int y, int x, const char* st, int fore, int back)
{
	int pair;
	for (pair = 0; pair < m_color_fore.size(); pair++)
		if (fore == m_color_fore[pair] && back == m_color_back[pair])
			break;
	if (pair == m_color_fore.size() && pair < COLOR_PAIRS -1)
	{
		init_pair(pair+1, fore, back);
		m_color_fore.push_back(fore);
		m_color_back.push_back(back);
	}

	attron(COLOR_PAIR(pair+1));
	mvprintw(y,x,st);
	attroff(COLOR_PAIR(pair+1));
}

Object*	Ui::find()
{
	int act = m_current_info;
	for (int i = 0; i < m_player_info[act]->objects.size(); i++)
		if (m_player_info[act]->objects[i]->id() == m_chosen_object)
			return m_player_info[act]->objects[i];
	return NULL;
}


void Ui::make_move(int x, int y)
{
	Object* a = find();
	if (a && a->owner() != m_player_id)
	{
		m_current_message = "Más vakarcsot csak úgy irányithatsz, ha varázslattal kényszerited.";
		return;
	}
	
	if (m_current_round >= m_player_info[m_current_info]->round + 10 && m_current_info == m_player_info.size()-1)
	{
		m_current_message = "10 lépésnél többet nem adhatsz meg!";
		return;
	}
	if (m_current_info < m_player_info.size()-1)
	{
		m_current_message = "Nem módoszíthasz a múlton. (Na jó, mégis, de ahhoz spéci varázslat kell.)";
		return;
	}

	if (a)
	{
		if (x == 0 && y == 0)
		{
			a->future_step[m_current_round - m_player_info[m_current_info]->round] = "";
			a->future_x[m_current_round - m_player_info[m_current_info]->round] = x;
			a->future_y[m_current_round - m_player_info[m_current_info]->round] = y;
			m_current_round++;
		}
		else
		{
			a->future_step[m_current_round - m_player_info[m_current_info]->round] = "move";
			a->future_x[m_current_round - m_player_info[m_current_info]->round] = x;
			a->future_y[m_current_round - m_player_info[m_current_info]->round] = y;
			m_current_round++;
		}
	}
	else
		m_current_message = "Nem jelöltél ki egy Vakarcsot sem. Használd a TAB-ot!";
}

void Ui::message(std::string str)
{
	m_current_info = m_player_info.size();
	m_current_info_string = 0;
	vector<string>* r = tokenize(str,'$');
	for (int i = 0; i < r->size(); i++)
	{
		PlayerInfo* player = new PlayerInfo;
		m_player_info.push_back(player);
		vector<string>* t = tokenize((*r)[i],'#');
		player->round = atol((*t)[0].c_str());
		if ((*t)[1] == "1")
			m_game_over = true;
//		m_last_round = player->round;

		vector<string>* s = tokenize((*t)[2],'^');
		for (int j = 0; j < s->size(); j++)
		{
			player->strings.push_back((*s)[j]);
		}
		delete s;
		
		player->map_for_player = new int[m_maxx*m_maxy];
		
		s = tokenize((*t)[3],'^');
		for (int j = 0; j < s->size(); j++)
		{
			player->map_for_player[j] = atoi((*s)[j].c_str());
		}
		delete s;
		
		s = tokenize((*t)[4],'^');
		for (int j = 0; j < s->size(); j++)
		{
			Object* toload = new Object((*s)[j]);
			player->objects.push_back(toload);
		}

		s = tokenize((*t)[5],'^');
		player->score = new int[s->size()];
		for (int j = 0; j < s->size(); j++)
		{
			player->score[j] = atoi((*s)[j].c_str());
		}

		delete s;

		delete t;
	}
	delete r;
	m_sent_positions.push_back(m_player_info.size()-1);
}

void Ui::help()
{
	clear();
	vector<string>* s = tokenize(m_t_help,'\n',true);
	for (int i = 0; i < s->size(); i++) // Ehh. We need all this stuff because of PDcurses.
		mvprintw(i,0,(*s)[i].c_str());
//	mvprintw(0,0,m_t_help.c_str());
/*	mvprintw(0,0,"Fontosabb parancsok:\n");
	mvprintw(2,0,"h           - segítség\n");
	mvprintw(3,0,"nyilak      - adott Vakarcs mozgatása\n");
	mvprintw(4,0,"TAB         - másik Vakarcs kiválasztása\n");
	mvprintw(5,0,"PgUp, PgDn  - elore/hátra tekerés az idöben\n");
	mvprintw(6,0,"F12         - lépéssorozat elküldése a szerverre\n");
	mvprintw(7,0,"ctrl-c      - kilépés\n");
*/
	delete s;
	refresh();
	getch();
}
void Ui::credits()
{
	clear();
	mvprintw(0,0,"Credits:");
	mvprintw(3,0,"Móra Péter (morapeter@gmail.com)  -  code, game, design");
	mvprintw(4,0,"Móra Gábor (morgab@gmail.com)     -  game, design");
	mvprintw(8,0,"Copyright 2008 Móra Péter");
	refresh();
	getch();
}

std::string Ui::play(void)
{
	int ch;
	MEVENT event;

	initscr ();             /* Belepes curses modba  */
	curs_set(0);
	start_color();
//	init_pair(1, COLOR_RED, COLOR_YELLOW);
	raw ();                 /* A soronkenti buffereles letiltasa */
	keypad (stdscr, TRUE);  /* Engedelyezzuk a funkciobillentyuk figyeleset */
	noecho ();              /* ne legyen visszhangja a getch () -nak */
	//	mousemask(ALL_MOUSE_EVENTS, NULL);

	int x,y;
//	getmaxyx( stdscr,x,y);
//	x /=2;
//	y /=2;
//	mvprintw(0,0,"F12-re elküldi a szervernek a lépéseket!\n");

	m_mode = 0;
	int act = m_current_info;
	if (!find() && m_player_info[act]->objects.size())
		m_chosen_object = m_player_info[act]->objects[0]->id();
	m_current_message = "F12-re küldi a szerverre a lépéseket! Segítség: h. Jó játékot!\n";
	if (m_sent_positions.size() > 1)
	{
		halfdelay(5);
		m_current_info = m_sent_positions[m_sent_positions.size()-2];
		m_current_info_string = 0;
		m_current_round = m_player_info[m_current_info]->round;
		if (m_current_info_string < m_player_info[m_current_info]->strings.size())
			m_current_message = m_player_info[m_current_info]->strings[m_current_info_string];
		draw_map();
		char ch = ERR;
		while (m_current_round < m_player_info[m_current_info]->round || m_current_info+1 < m_player_info.size() || (m_current_info+1 == m_player_info.size() && m_current_info_string+1 < m_player_info[m_current_info]->strings.size() ))
		{
			next_step();
			//sleep(1);
			if (ch == ERR)
				ch = getch();
			draw_map();
		}
		nocbreak();
		raw();
	}
	draw_map();
	//	mvprintw(1,0,"Koordináta: %d, %d      ",x,y);
//	attron(A_BOLD | COLOR_PAIR(1)); mvprintw(x,y, "X"); attroff(A_BOLD | COLOR_PAIR(1));
//	refresh();
	while((ch = getch()) != KEY_F(12) || m_game_over)
	{	
		int lower, upper, act;
		int old;
		m_current_message = "";
		if (m_game_over && ch == KEY_F(12))
			m_current_message = "A játéknak vége, kilépés Ctrl-C -vel.";
		Object* ob = find();
//		m_current_message = itostr(ch);
		switch(ch)
		{
			case KEY_F(2):
				if (++m_wall_type == 3)
					m_wall_type = 0;
			case 27:	//ESC
				m_mode =0;
				break;
			case 3:	//Ctrl-C
				endwin();
				if (m_game_over)
				{
					cout << "\n";
					return "quit";
				}
				return "exit";
				break;
			case KEY_LEFT:
				if(m_game_over)
				{
					m_current_message = "A játéknak vége.";
					break;
				}
				if(m_mode == 2)
				{
					set_ability(-1,0);
					break;
				}
				if(m_mode == 3)
				{
					m_set_ability_x--;
					break;
				}
				make_move(-1,0);
				break;
			case KEY_RIGHT:
				if(m_game_over)
				{
					m_current_message = "A játéknak vége.";
					break;
				}
				if(m_mode == 2)
				{
					set_ability(1,0);
					break;
				}
				if(m_mode == 3)
				{
					m_set_ability_x++;
					break;
				}
				make_move(1,0);
				break;
			case KEY_UP:
				if(m_game_over)
				{
					m_current_message = "A játéknak vége.";
					break;
				}
				if(m_mode == 2)
				{
					set_ability(0,-1);
					break;
				}
				if(m_mode == 3)
				{
					m_set_ability_y--;
					break;
				}
				make_move(0,-1);
				break;
			case KEY_DOWN:
				if(m_game_over)
				{
					m_current_message = "A játéknak vége.";
					break;
				}
				if(m_mode == 2)
				{
					set_ability(0,1);
					break;
				}
				if(m_mode == 3)
				{
					m_set_ability_y++;
					break;
				}
				make_move(0,1);
				break;
			case ' ':
			case '.':
				if(m_game_over)
				{
					m_current_message = "A játéknak vége.";
					break;
				}
				if(m_mode == 2)
				{
					m_mode = 0;
					break;
				}
				if(m_mode == 3)
				{
					set_ability(m_set_ability_x,m_set_ability_y);
					break;
				}
				make_move(0,0);
				break;
			case 9://TAB
				if(m_mode == 2 || m_mode ==3)
				{
					m_mode = 0;
				}
				old = m_chosen_object;
				act = m_current_info;
				lower = m_chosen_object;
				upper = m_chosen_object;
				for (int i = 0; i < m_player_info[act]->objects.size(); i++)
				{
					int q = m_player_info[act]->objects[i]->id();
					if ((q < upper && q > m_chosen_object) ||
						       (q > upper && upper == m_chosen_object))
							       upper = q;
					if (q < lower)
						lower = q;
		     	       }
				if (upper == m_chosen_object)
					m_chosen_object = lower;
				else
					m_chosen_object = upper;
				ob = find();
				if (ob)
				{
						m_vak_red = ob->red();
						m_vak_green = ob->green();
						m_vak_blue = ob->blue();
						m_vak_white = ob->white();
				}
				break;
			case 351://SHIFT TAB
			case 353://SHIFT TAB
				if(m_mode == 2 || m_mode ==3)
				{
					m_mode = 0;
				}
				old = m_chosen_object;
				act = m_current_info;
				lower = m_chosen_object;
				upper = m_chosen_object;
				for (int i = 0; i < m_player_info[act]->objects.size(); i++)
				{
					int q = m_player_info[act]->objects[i]->id();
					if ((q > upper && q < m_chosen_object) ||
						       (q < upper && upper == m_chosen_object))
							       upper = q;
					if (q > lower)
						lower = q;
		     	       }
				if (upper == m_chosen_object)
					m_chosen_object = lower;
				else
					m_chosen_object = upper;
				ob = find();
				if (ob)
				{
						m_vak_red = ob->red();
						m_vak_green = ob->green();
						m_vak_blue = ob->blue();
						m_vak_white = ob->white();
				}
				break;
			case KEY_BACKSPACE:
			case 8:
				if (m_mode > 1)
					m_mode = 0;
				m_current_info = m_sent_positions.back();
				m_current_info_string = 0;
				m_current_round = m_player_info[m_current_info]->round;
				break;
			case KEY_NPAGE:
				if(m_mode == 2 || m_mode == 3)
				{
					m_mode = 0;
				}
				next_step();
				break;
			case KEY_PPAGE:
				if(m_mode == 2 || m_mode == 3)
				{
					m_mode = 0;
				}
				prev_step();
				break;
			case 10://KEY_ENTER:
			case 13:
				if(m_mode == 2)
				{
					m_mode = 0;
				}
				if(m_mode == 3)
				{
					set_ability(m_set_ability_x,m_set_ability_y);
					break;
				}
				if (m_mode == 1)
				{
					m_mode = 0;
				}
				else
				{
					ob=find();
					if (!ob)
					{
						m_current_message = "Használd a TAB-ot, hogy kijelölj egy vakarocsot!";
						m_vak_red = 1;
						m_vak_green = 0;
						m_vak_blue = 0;
						m_vak_white = 0;
					}
					else
					{
						m_vak_red = ob->red();
						m_vak_green = ob->green();
						m_vak_blue = ob->blue();
						m_vak_white = ob->white();
					}
					m_mode = 1;
				}
				break;
/*			case KEY_MOUSE:
				if(getmouse(&event) == OK)
				{
				if(event.bstate & BUTTON1_CLICKED)
				{
					y = event.x;
					x = event.y;
				}
				
				}
				break;*/
			case 'h':
			case 'H':
			case KEY_F(1):
				if(m_mode == 2)
				{
					m_mode = 0;
				}
				help();
				break;
			case 'c':
			case 'C':
				if(m_mode == 2)
				{
					m_mode = 0;
				}
				credits();
				break;
			case 'q':
			case 'Q':
				if (m_vak_red < MAX_VAKARCS)
					m_vak_red++;
				m_vak_white = 0;
				break;
			case 'w':
			case 'W':
				if (m_vak_green < MAX_VAKARCS)
					m_vak_green++;
				m_vak_white = 0;
				break;
			case 'e':
			case 'E':
				if (m_vak_blue < MAX_VAKARCS)
					m_vak_blue++;
				m_vak_white = 0;
				break;
			case 'r':
			case 'R':
				if (m_vak_white < 2*MAX_VAKARCS)
					m_vak_white++;
				m_vak_red = m_vak_green = m_vak_blue = 0;
				break;
			case 'a':
			case 'A':
				if (m_vak_red > 0)
					m_vak_red--;
				break;
			case 's':
			case 'S':
				if (m_vak_green > 0)
					m_vak_green--;
				break;
			case 'd':
			case 'D':
				if (m_vak_blue > 0)
					m_vak_blue--;
				break;
			case 'f':
			case 'F':
				if (m_vak_white > 0)
					m_vak_white--;
				break;
			case '1':
			case '2':
			case '3':
			case '4':
			case '5':
			case '6':
			case '7':
			case '8':
			case '9':
			case '0':
			case '`':
				if(m_game_over)
				{
					m_current_message = "A játéknak vége.";
					break;
				}
				if(m_mode == 2 || m_mode == 3)
				{
					m_mode = 0;
				}
				if (ob && ob->owner() != m_player_id)
				{
					m_current_message = "Más vakarcsot csak úgy irányithatsz, ha varázslattal kényszerited.";
					break;
				}

				if (m_current_round >= m_player_info[m_current_info]->round + 10 && m_current_info == m_player_info.size()-1)
				{
					m_current_message = "10 lépésnél többet nem adhatsz meg!";
					break;
				}
				if (m_current_info < m_player_info.size()-1)
				{
					m_current_message = "Nem módoszíthasz a múlton. (Na jó, mégis, de ahhoz spéci varázslat kell.)";
					break;
				}

				if (ob)
				{
					if (ch == '0' || ch == '`')
					{
						ob->future_step[m_current_round - m_player_info[m_current_info]->round] = "fuse";
						m_current_round++;
					}
					else
					{
						if (m_abilities_types[ch -'1'] == 'D') //Direction
						{
							m_set_ability_round = m_current_round - m_player_info[m_current_info]->round;
							m_set_ability_string = string("ability")+itostr(ch-'0');
							m_mode = 2;
						}
						if (m_abilities_types[ch -'1'] == 'C')//Coordinates
						{
							m_set_ability_round = m_current_round - m_player_info[m_current_info]->round;
							m_set_ability_string = string("ability")+itostr(ch-'0');
							m_mode = 3;
							m_set_ability_x = 0;
							m_set_ability_y = 0;
							if (ob)
								position(ob,m_set_ability_x,m_set_ability_y);
							m_set_ability_from_x = m_set_ability_x;
							m_set_ability_from_y = m_set_ability_y;
						}
						if (m_abilities_types[ch -'1'] == 'A')//Activated
						{
							ob->future_step[m_current_round - m_player_info[m_current_info]->round] = string("ability")+itostr(ch-'0');
							ob->future_x[m_current_round - m_player_info[m_current_info]->round] = 0;
							ob->future_y[m_current_round - m_player_info[m_current_info]->round] = 0;
							m_current_round++;
						}
					}
				}
				else
					m_current_message = "Nem jelöltél ki egy Vakarcsot sem. Használd a TAB-ot!";
				break;


		}
		if (m_mode == 2)
			m_current_message = "A nyilakkal adj meg egy irányt!";
		if (m_mode == 3)
			m_current_message = "A nyilakkal jelölj ki egy koordinátát. Enterrel tudod elfogadni.";
		draw_map();
	}
	endwin();

	//generating message to server
	string m;
	for (int i = 0; i < 10; i++)
	{
		for (int j = 0; j < m_player_info.back()->objects.size(); j++)
		{
			Object* ob = m_player_info.back()->objects[j];
			m += itostr(ob->id()) + " " + ob->future_step[i] + " " + itostr(ob->future_x[i]) + " " + itostr(ob->future_y[i]) + " ";
//			if (j < m_player_info.back()->objects.size() -1)
				m+="#";
		}	
//		if (i < 9)
			m += "$";
	}
//	cout << m << "\n";
	return std::string(m);
};

void	Ui::set_ability(int x, int y)
{
	Object* ob = find();
	if (ob)
	{
		ob->future_step[m_set_ability_round]=m_set_ability_string;
		ob->future_x[m_set_ability_round] = x; ob->future_y[m_set_ability_round] = y;
		m_current_round++;
	}
	m_mode = 0;
}
void	Ui::draw_map()
{
	clear();
	if (m_mode == 3)
		m_current_message = string("Lövési távolság: ")+itostr(MAX(abs(m_set_ability_x-m_set_ability_from_x),abs(m_set_ability_y-m_set_ability_from_y)));
	m_color_fore.clear();
	m_color_back.clear();
	int mark_x,mark_y,mark_round;
	mark_round = -1;
	if (m_current_message.size() > 2 && m_current_message.substr(0,2) == "--")
	{
		string s = m_current_message;
//		m_current_message = m_current_message.substr(m_current_message.rfind("--")+2,m_current_message.size());
		mvprintw(0,0,m_current_message.substr(m_current_message.rfind("--")+2,m_current_message.size()).c_str());
		s = s.substr(9,s.rfind("--")-10);
		vector<string>* t = tokenize(s,',',true);
		if (t->size() == 3)
		{
			mark_x = atol((*t)[0].c_str());
			mark_y = atol((*t)[1].c_str());
			mark_round = atol((*t)[2].c_str());
		}
		delete t;
	}
	else
		mvprintw(0,0,m_current_message.c_str());
	
	int act = m_current_info;
	int *map = m_player_info[act]->map_for_player;
	string info;
	info = itostr(m_current_round) + "/" + itostr(m_player_info.back()->round) + ". lépés";

	m_info_print_line = 0;
	m_info_print_pos = 0;

	if (m_current_round >= m_player_info[m_current_info]->round && m_current_info == m_player_info.size()-1) // It's in the future
		attron(A_BOLD);
	info_print(info.c_str(),COLOR_WHITE, COLOR_BLACK);
	if (m_current_round >= m_player_info[m_current_info]->round && m_current_info == m_player_info.size()-1) // It's in the future
		attroff(A_BOLD);

	info_print(", állás: ");

	for (int i = 0; i < m_number_of_players; i++)
	{
		if (i == m_player_id)
			attron(A_BOLD);
		info_print( itostr(m_player_info[act]->score[i]).c_str() );
		if (i == m_player_id)
			attroff(A_BOLD);
		if (i < m_number_of_players-1)
			info_print("/");
	}
	info_print(", ");

	Object* ob = NULL;
	int obx = -1;
	int oby = -1;

	if (m_mode == 0 || m_mode == 2 || m_mode == 3)
	{
		int* coll = new int [m_maxx*m_maxy]; //for detecting collisions
		for (int x = 0; x < m_maxx; x++)
			for (int y = 0; y < m_maxy; y++)
			{
				if (m_mode == 3 && m_set_ability_x == x && m_set_ability_y == y)
					attron(A_BLINK | A_REVERSE);
				int n = 0;
				switch (map[y*m_maxx + x])
				{
					case 0:
						mvprintw(y+MAP_OFFSET,x," ");
//						mvaddch(y+MAP_OFFSET,x,ACS_CKBOARD);
						break;
					case MAP_WALL:
						//mvprintw(y+MAP_OFFSET,x,"*");
						if (m_wall_type == 1)
						{
							if (x < m_maxx-1 && map[y*m_maxx + x+1] == MAP_WALL)
								n += 1;
							if (y > 0 && map[(y-1)*m_maxx + x] == MAP_WALL)
								n += 10;
							if (x >0  && map[y*m_maxx + x-1] == MAP_WALL)
								n += 100;
							if (y < m_maxy-1 && map[(y+1)*m_maxx + x] == MAP_WALL)
								n += 1000;
							switch(n)
							{
								case 0:
									mvaddch(y+MAP_OFFSET,x,ACS_DIAMOND);
									break;
								case 1:
								case 100:
								case 101:
									mvaddch(y+MAP_OFFSET,x,ACS_HLINE);
									//		mvaddch(y+MAP_OFFSET,x,ACS_DIAMOND);
									break;
								case 10:
								case 1000:
								case 1010:
									mvaddch(y+MAP_OFFSET,x,ACS_VLINE);
									//		mvaddch(y+MAP_OFFSET,x,ACS_DIAMOND);
									break;
								case 11:
									mvaddch(y+MAP_OFFSET,x,ACS_LLCORNER);
									break;
								case 110:
									mvaddch(y+MAP_OFFSET,x,ACS_LRCORNER);
									break;
								case 1100:
									mvaddch(y+MAP_OFFSET,x,ACS_URCORNER);
									break;
								case 1001:
									mvaddch(y+MAP_OFFSET,x,ACS_ULCORNER);
									break;
								case 111:
									mvaddch(y+MAP_OFFSET,x,ACS_BTEE);
									break;
								case 1110:
									mvaddch(y+MAP_OFFSET,x,ACS_RTEE);
									break;
								case 1101:
									mvaddch(y+MAP_OFFSET,x,ACS_TTEE);
									break;
								case 1011:
									mvaddch(y+MAP_OFFSET,x,ACS_LTEE);
									break;
								case 1111:
									mvaddch(y+MAP_OFFSET,x,ACS_PLUS);
									break;
							}
						}
						if (m_wall_type == 2)
							mvaddch(y+MAP_OFFSET,x,ACS_DIAMOND);
						if (m_wall_type == 0)
							mvaddch(y+MAP_OFFSET,x,ACS_CKBOARD);
						break;
					case MAP_EMPTY:
						//mvprintw(y+MAP_OFFSET,x,"`");
						mvaddch(y+MAP_OFFSET,x,ACS_BULLET);
						break;
					case MAP_DESTROYED:
						mvprintw(y+MAP_OFFSET,x,"#");
						break;
				}
				if (map[y*m_maxx+x] >= MAP_BASE)
					mvprintw(y+MAP_OFFSET,x,"O");
				if (m_mode == 3 && m_set_ability_x == x && m_set_ability_y == y)
					attroff(A_BLINK | A_REVERSE);
				coll[y*m_maxx+x] = 0;
			}
		for (int i = 0; i < m_player_info[act]->objects.size(); i++)
		{
			if(m_player_info[act]->objects[i]->id() == m_chosen_object)
			{
				ob = m_player_info[act]->objects[i];
				attron(A_BOLD); 
			}

			int x, y;
			position(m_player_info[act]->objects[i],x,y);
			if(m_player_info[act]->objects[i]->id() == m_chosen_object)
				attron(A_BOLD);
			if (x == obx && y == oby)
				attron(A_BOLD);
			if (m_mode == 3 && m_set_ability_x == x && m_set_ability_y == y)
				attron(A_BLINK | A_REVERSE);
			if (x >= 0 && y >= 0 && x < m_maxx && y < m_maxy && coll[y*m_maxx+x]++ > 0)
			{
				if (coll[y*m_maxx+x] < 10)
					mvprintw(y+MAP_OFFSET,x, itostr(coll[y*m_maxx+x]).c_str()); 
				else
					mvprintw(y+MAP_OFFSET,x, "0"); 
			}
			else
			{
				if (m_mode == 0 && m_player_info[act]->objects[i]->owner() != m_player_id)
					attron(A_REVERSE);
			//	mvprintw(y+MAP_OFFSET,x, "%c", m_player_info[act]->objects[i]->name()[0]); 
				int fore = COLOR_WHITE;
				int back = COLOR_BLACK;
				switch( (m_player_info[act]->objects[i]->red() >0?1:0 )*1000+(m_player_info[act]->objects[i]->green()>0?1:0)*100+(m_player_info[act]->objects[i]->blue()>0?1:0)*10 +(m_player_info[act]->objects[i]->white()>0?1:0) )
				{
					case	1000:
						fore = COLOR_RED;
						break;
					case	100:
						fore = COLOR_GREEN;
						break;
					case	10:
						fore = COLOR_BLUE;
						break;
					case	1100:
						fore = COLOR_GREEN;
						back = COLOR_RED;
						break;
					case	1010:
						fore = COLOR_RED;
						back = COLOR_BLUE;
						break;
					case	110:
						fore = COLOR_GREEN;
						back = COLOR_BLUE;
						break;
					case	1110:
						fore = COLOR_YELLOW;
						break;
				}
				color_print(y+MAP_OFFSET, x, m_player_info[act]->objects[i]->name().substr(0,1).c_str(), fore, back);
				color_print(y+MAP_OFFSET, x, m_player_info[act]->objects[i]->name().substr(0,1).c_str(), fore, back);
				if (m_mode == 0 && m_player_info[act]->objects[i]->owner() != m_player_id)
					attroff(A_REVERSE);
			}
			if (x == obx && y == oby)
				attroff(A_BOLD);
			if (m_mode == 3 && m_set_ability_x == x && m_set_ability_y == y)
				attroff(A_BLINK | A_REVERSE);

			if(m_player_info[act]->objects[i]->id() == m_chosen_object)
			{
				obx = x; oby = y;
				attroff(A_BOLD);
			}
		}
		delete[] coll;
		if (mark_round != -1)
		{
			for (int x = 0; x < m_maxx; x++)
				for (int y= 0; y < m_maxy; y++)
					if ( (int)sqrt((x-mark_x)*(x-mark_x)+(y-mark_y)*(y-mark_y)) == mark_round)
							mvchgat(y+MAP_OFFSET,x,1,A_REVERSE|A_BLINK,0,NULL);
		}
	}
	ob=find();
	if (m_mode == 1)
	{
		int line = MAP_OFFSET;
//		if (ob)
//		{
//			mvprintw(line++,0,(string("A vakarcs neve: ") + ob->name()).c_str());
//			if (ob->red() == m_vak_red && ob->green() == m_vak_green && ob->blue() == m_vak_blue && ob->white() == m_vak_white)
//				attron(A_BOLD);
//		}
		int x,y;
		getmaxyx(stdscr,y,x);
		attron(A_BOLD);
		mvprintw(line++,0,"Saját vakarcsok:");
		attroff(A_BOLD);
		int row = 0;
		bool enemy = false;
		for (int i = 0; i < m_player_info[m_current_info]->objects.size(); i++)
			if (m_player_info[m_current_info]->objects[i]->owner() == m_player_id)
			{
				Object* o = m_player_info[m_current_info]->objects[i];
				if (o->red())
					attron(A_BOLD);
				color_print(line,row,itostr(o->red()).c_str(),COLOR_RED,COLOR_BLACK);
				if (o->red())
					attroff(A_BOLD);
				if (o->green())
					attron(A_BOLD);
				color_print(line,row+2,itostr(o->green()).c_str(),COLOR_GREEN,COLOR_BLACK);
				if (o->green())
					attroff(A_BOLD);
				if (o->blue())
					attron(A_BOLD);
				color_print(line,row+4,itostr(o->blue()).c_str(),COLOR_BLUE,COLOR_BLACK);
				if (o->blue())
					attroff(A_BOLD);
				if (o->white())
					attron(A_BOLD);
				color_print(line,row+6,itostr(o->white()).c_str(),COLOR_WHITE,COLOR_BLACK);
				if (o->white())
					attroff(A_BOLD);
				if (m_chosen_object == o->id())
					attron(A_BOLD);
				int xx,yy;
				mvprintw(line,row+8,o->name().c_str());
				getyx(stdscr,yy,xx);
//				row += xx-row+3;
				row += 20;
				if (m_chosen_object == o->id())
					attroff(A_BOLD);
				if(x < row+20)
				{
					line++;
					row = 0;
				}
			}
			else
				enemy = true;
		line++;
		line++;
		if (enemy)
		{
			attron(A_BOLD);
			mvprintw(line++,0,"Ellenséges észlelt vakarcsok:");
			attroff(A_BOLD);
			int row = 0;
			for (int i = 0; i < m_player_info[m_current_info]->objects.size(); i++)
				if (m_player_info[m_current_info]->objects[i]->owner() != m_player_id)
				{
					Object* o = m_player_info[m_current_info]->objects[i];
					if (o->red())
						attron(A_BOLD);
					color_print(line,row,itostr(o->red()).c_str(),COLOR_RED,COLOR_BLACK);
					if (o->red())
						attroff(A_BOLD);
					if (o->green())
						attron(A_BOLD);
					color_print(line,row+2,itostr(o->green()).c_str(),COLOR_GREEN,COLOR_BLACK);
					if (o->green())
						attroff(A_BOLD);
					if (o->blue())
						attron(A_BOLD);
					color_print(line,row+4,itostr(o->blue()).c_str(),COLOR_BLUE,COLOR_BLACK);
					if (o->blue())
						attroff(A_BOLD);
					if (o->white())
						attron(A_BOLD);
					color_print(line,row+6,itostr(o->white()).c_str(),COLOR_WHITE,COLOR_BLACK);
					if (o->white())
						attroff(A_BOLD);
					if (m_chosen_object == o->id())
						attron(A_BOLD);
					mvprintw(line,row+8,o->name().c_str());
					row += 8+o->name().size()+3;
					if (m_chosen_object == o->id())
						attroff(A_BOLD);
					if(x < row+20)
					{
						line++;
						row = 0;
					}
				}
			line++;
			line++;
		}
//		attron(A_BOLD);
		if (m_vak_red)
			attron(A_BOLD);
		color_print(line,0,itostr(m_vak_red).c_str(),COLOR_RED,COLOR_BLACK);
		if (m_vak_red)
			attroff(A_BOLD);
		if (m_vak_green)
			attron(A_BOLD);
		color_print(line,2,itostr(m_vak_green).c_str(),COLOR_GREEN,COLOR_BLACK);
		if (m_vak_green)
			attroff(A_BOLD);
		if (m_vak_blue)
			attron(A_BOLD);
		color_print(line,4,itostr(m_vak_blue).c_str(),COLOR_BLUE,COLOR_BLACK);
		if (m_vak_blue)
			attroff(A_BOLD);
		if (m_vak_white)
			attron(A_BOLD);
		color_print(line++,6,itostr(m_vak_white).c_str(),COLOR_WHITE,COLOR_BLACK);
		if (m_vak_white)
			attroff(A_BOLD);
//		color_print(line,0,itostr(m_vak_red).c_str(),COLOR_RED,COLOR_BLACK);
//		color_print(line,2,itostr(m_vak_green).c_str(),COLOR_GREEN,COLOR_BLACK);
//		color_print(line,4,itostr(m_vak_blue).c_str(),COLOR_BLUE,COLOR_BLACK);
//		color_print(line++,6,itostr(m_vak_white).c_str(),COLOR_WHITE,COLOR_BLACK);
//		attroff(A_BOLD);
		mvprintw(line++,0,"q/a, w/s, e/d, r/f gombokkal megnézheted");
		mvprintw(line++,0,"más összeállítású vakarcsok képességeit.");
		line++;
		if (ob)
		{
			Ability* v = find_ability(m_vak_red, m_vak_green, m_vak_blue, m_vak_white);
			attron(A_BOLD);
			mvprintw(line,0,"0");
			mvprintw(line++,1,": egyesülés");
			attroff(A_BOLD);
			mvprintw(line++,1,"A vakarcs egyesül a vele egy mezön álló többi vakarccsal.");

			if (v)
			{
				for (int i = 0; i < v->ability.size(); i++)
				{
					int row = 0;
					attron(A_BOLD);
					if (m_abilities_types[v->id[i]-1] != 'S')	// NOT STATIC
					{
						mvprintw(line,0,(itostr(v->id[i])).c_str());
						mvprintw(line,1,": ");
						row = 3;
					}
					int tr = find_translate(v->ability[i]);
					if (tr == -1)
					{
						mvprintw(line++,row,(v->ability[i]+", "+m_t_parameter+": "+itostr(v->parameter[i])+", "+m_t_cooldown + ": " + itostr(v->cooldown[i])).c_str());
						mvprintw(line++,row," ???");
						attroff(A_BOLD);
						line++;

					}
					else
					{
						mvprintw(line,row,m_translate[tr]->longname.c_str());
						attroff(A_BOLD);
						if (m_translate[tr]->parameter != "-")
							printw((string(", ") + m_translate[tr]->parameter + ": " + itostr(v->parameter[i]) ).c_str());
						if (m_abilities_types[v->id[i]-1] != 'S')	// NOT STATIC
							printw((string(", ")+m_t_cooldown + ": " + itostr(v->cooldown[i]) + " lépés").c_str());
						line++;
						desc_print(&line,m_translate[tr]->description);
						//mvprintw(line++,1,m_translate[tr]->description.c_str());
						line++;
					}
				}
			}
			else
				if (m_vak_red == 0 && m_vak_green ==0 && m_vak_blue == 0 && m_vak_white == 0)
					
					mvprintw(line++,0,"A gombokkal adj meg meg egy kombinációt.");
				else
					mvprintw(line++,0,"Hiba: nem találom az adott kombinációhoz tartozó képességeket");
			line++;


		}
		
	}
	if (ob)
	{
		info_print(ob->name().c_str());
		info_print(", vakarcsok: ");
		if (ob->red())
			attron(A_BOLD);
		info_print(itostr(ob->red()).c_str(),COLOR_RED,COLOR_BLACK);
		if (ob->red())
			attroff(A_BOLD);
		info_print(" ");
		if (ob->green())
			attron(A_BOLD);
		info_print(itostr(ob->green()).c_str(),COLOR_GREEN,COLOR_BLACK);
		if (ob->green())
			attroff(A_BOLD);
		info_print(" ");
		if (ob->blue())
			attron(A_BOLD);
		info_print(itostr(ob->blue()).c_str(),COLOR_BLUE,COLOR_BLACK);
		if (ob->blue())
			attroff(A_BOLD);
		info_print(" ");
		if (ob->white())
			attron(A_BOLD);
		info_print(itostr(ob->white()).c_str(),COLOR_WHITE,COLOR_BLACK);
		if (ob->white())
			attroff(A_BOLD);
		info_print(" ");
		m_info_print_pos = 0;
		m_info_print_line = 1;
		if (ob->owner() == m_player_id)
		{
			info_print("Parancsok: ");
			int p = 0;
			while(m_sent_positions.size() -1 > p && m_sent_positions[p+1] <= m_current_info)
				p++;
			for (int i = 0; i < m_player_info[m_sent_positions[p]]->objects.size(); i++)
				if (ob->id() == m_player_info[m_sent_positions[p]]->objects[i]->id())
					ob = m_player_info[m_sent_positions[p]]->objects[i];
			for (int i = 0; i < 10; i++)
			{
				if (m_current_round - m_player_info[m_sent_positions[p]]->round == i)
					attron(A_BOLD);
				bool found = false;
				if (ob->future_step[i] == "")
				{
					if (m_current_round >= m_player_info[m_current_info]->round && m_current_info == m_player_info.size()-1) // It's in the future
						info_print("üres ");
					else
						info_print("pihen ");
					found = true;

				}
				if (ob->future_step[i] == "move")
				{
					switch(ob->future_x[i]+ob->future_y[i]*10)
					{
						case 10:
							info_print("LE ");
							break;
						case 1:
							info_print("JOBB ");
							break;
						case -1:
							info_print("BAL ");
							break;
						case -10:
							info_print("FEL ");
							break;
					}
					found = true;
				}
				if (ob->future_step[i].substr(0,7) == "ability")
				{
					Object* ob_color = NULL;
					for (int e = 0; e < m_player_info[MIN(m_sent_positions[p]+i,m_player_info.size()-1)]->objects.size(); e++)
						if (m_player_info[MIN(m_sent_positions[p]+i,m_player_info.size()-1)]->objects[e]->id() == m_chosen_object)
							ob_color = m_player_info[MIN(m_sent_positions[p]+i,m_player_info.size()-1)]->objects[e];
					int q = ob->future_step[i][7] - '1';
					int a = 0;
					Ability* v = NULL;
					if (ob_color)
						v = find_ability(ob_color->red(),ob_color->green(),ob_color->blue(),ob_color->white());

					info_print((itostr(q+1)+string("-")).c_str());
					int w = -1;
					if (v)
						w = v->number[q+1];
					if (v && w != -1)
					{
						int tr = find_translate(v->ability[w]);
						if (tr == -1)
							info_print(v->ability[w].c_str());
						else
							info_print(m_translate[tr]->shortname.c_str());
					}

					if (m_abilities_types[q] == 'D')
						switch (ob->future_x[i] + 10*ob->future_y[i] )
						{
							case -1:
								info_print("<");
								break;
							case 1:
								info_print(">");
								break;
							case -10:
								info_print("^");
								break;
							case 10:
								info_print("v");
								break;
						}
					if (m_abilities_types[q] == 'C')
					{
						info_print((string("(")+itostr(ob->future_x[i]) + "," +  itostr(ob->future_y[i])+")").c_str() );
						if (m_current_round - m_player_info[m_sent_positions[p]]->round == i && m_mode == 0)
						{
							//	mvprintw(ob->future_y[i]+MAP_OFFSET,ob->future_x[i],"X");
							//	move(ob->future_y[i]+MAP_OFFSET,ob->future_x[i]);
							mvchgat(ob->future_y[i]+MAP_OFFSET,ob->future_x[i],1,A_REVERSE|A_BLINK,0,NULL);
						}
					}
					info_print(" ");

					found = true;
				}
				if (ob->future_step[i] == "fuse")
				{
					info_print("egyesül ");
					found  = true;
				}
				if (!found)
					info_print("??? ");
				if (m_current_round - m_player_info[m_sent_positions[p]]->round == i)
					attroff(A_BOLD);
			}
		}
	}

/*		mvprintw(10,1,itostr(m_current_round).c_str());
		mvprintw(11,1,itostr(m_current_info).c_str());
		mvprintw(12,1,itostr(m_current_info_string).c_str());
		mvprintw(12,5,itostr(m_player_info[m_current_info]->strings.size()-1).c_str());
		mvprintw(13,1,itostr(m_player_info[m_current_info]->round).c_str());
	for (int i = 0; i < m_sent_positions.size(); i++)
		mvprintw(15+i,1,itostr(m_sent_positions[i]).c_str());
*/
/*
   mvaddch(2,1,ACS_ULCORNER);
   mvaddch(3,1,ACS_LLCORNER);
   mvaddch(4,1,ACS_URCORNER);
   mvaddch(5,1,ACS_LRCORNER);
   mvaddch(6,1,ACS_LTEE);
   mvaddch(7,1,ACS_RTEE);
   mvaddch(8,1,ACS_BTEE);
   mvaddch(9,1,ACS_TTEE);
   mvaddch(10,1,ACS_HLINE);
   mvaddch(11,1,ACS_VLINE);
   mvaddch(12,1,ACS_PLUS);
   mvaddch(13,1,ACS_S1);
   mvaddch(14,1,ACS_S9);
   mvaddch(15,1,ACS_DIAMOND);
   mvaddch(16,1,ACS_CKBOARD);
   mvaddch(17,1,ACS_DEGREE);
   mvaddch(18,1,ACS_PLMINUS);
   mvaddch(19,1,ACS_BULLET);
   mvaddch(20,1,ACS_S3);
   mvaddch(21,1,ACS_S7);
   mvaddch(22,1,ACS_BOARD);
   mvaddch(23,1,ACS_LANTERN);
   mvaddch(24,1,ACS_BLOCK);
  */ 
	refresh();
};

void	Ui::load_language(string str)
{
	FILE* f = fopen(str.c_str(),"r");
	if (!f)
	{
		cout << "Language file: " << str.c_str() << " is missing.\n";
		exit(1);
	}
	char s[1000];
	fscanf(f,"cooldown: ");
	fgets(s,1000,f);
	if (strlen(s))  s[strlen(s)-1] = 0;
	m_t_cooldown = s;
	cout << m_t_cooldown << "\n";
	fscanf(f,"parameter: %s\n",s);
	m_t_parameter = s;
	fscanf(f,"help:\n");
	fgets(s,1000,f);
	while(string(s) != "HELP_END\n")
	{
//		cout << strlen(s) << "\n";
		m_t_help += s;
		fgets(s,1000,f);
	}
	fscanf(f,"\n");
	while(!feof(f))
	{
		fscanf(f,"%s\n",s);
		if (feof(f))
			break;
		Translate* t = new Translate;
		t->name = s;
		fscanf(f,"shortname: ");
		fgets(s,1000,f);
		if (strlen(s))  s[strlen(s)-1] = 0;
		t->shortname = s;
		fscanf(f,"longname: ");
		fgets(s,1000,f);
		if (strlen(s))  s[strlen(s)-1] = 0;
		t->longname = s;
		fscanf(f,"parameter: ");
		fgets(s,1000,f);
		if (strlen(s))  s[strlen(s)-1] = 0;
		t->parameter = s;
		fscanf(f,"description: \n");
		fgets(s,1000,f);
		if (strlen(s))  s[strlen(s)-1] = 0;
		t->description = s;
		m_translate.push_back(t);
//		cout << t->name << "~" <<t->shortname << "~" <<  t->parameter << "~" << t->description <<  "\n"; 
		fscanf(f,"\n");
	}
}

int	Ui::find_translate(string str)
{
	for (int i = 0; i < m_translate.size(); i++)
		if (str == m_translate[i]->name)
			return i;
	return -1;
}

void	Ui::desc_print(int* line, string str)
{ 
	int x, y;
	getmaxyx( stdscr,y,x);

	int col = 1;
	string word;
	for (int i =0; i <= str.length(); i++)
	{
		if (i == str.length() || str[i] == ' ')
		{
			if (word.length()+col >= x && col != 1)
			{
				col = 1;
				(*line)++;
			}
			mvprintw(*line,col,word.c_str());
			int xx,yy;
			getyx(stdscr,yy,xx);
			col = xx+1;
			word = "";
		}
		else
			word += str[i];
	}
}
Ability*	Ui::find_ability(int red, int green, int blue, int white)
{
	int a = 0;
	for (;a<m_abilities.size();a++)
	{
		if (m_abilities[a]->red == red && m_abilities[a]->green == green && m_abilities[a]->blue == blue && m_abilities[a]->white == white)
			return m_abilities[a];
	}
	return NULL;
}


void	Ui::position(Object* ob, int &x, int &y)
{
	x = ob->x();
	y = ob->y();


	if (m_current_round > m_player_info[m_current_info]->round && m_current_info == m_player_info.size()-1) // It's in the future
	{
		for (int q = 0; q < m_current_round - m_player_info[m_current_info]->round; q++)
		{
			if (ob->future_step[q] == "move")
			{
				x += ob->future_x[q];
				y += ob->future_y[q];
			}
			if (ob->future_step[q].substr(0,7) == "ability")
			{
				int w = ob->future_step[q][7] - '0';
				Ability* v = find_ability(ob->red(),ob->green(),ob->blue(),ob->white());
				w = v->number[w];
				if (v && w != -1)
				{
					if(v->ability[w] == "blink")
					{
						x += ob->future_x[q]*v->parameter[w];
						y += ob->future_y[q]*v->parameter[w];
					}
				}
			}
		}
	}

}

void	Ui::next_step()
{
	m_current_message = "";
	if (m_current_round >= m_player_info[m_current_info]->round + 10 && m_current_info == m_player_info.size()-1)
	{
		m_current_message = "10 lépésnél többet nem adhatsz meg!";
	}
	else
	{
		if (m_current_info == m_player_info.size()-1 && (m_current_round > m_player_info[m_current_info]->round || m_player_info[m_current_info]->strings.size() <= m_current_info_string+1))
		{
			m_current_round++;
		}
		else
		{
			if (m_current_info_string+1 >= m_player_info[m_current_info]->strings.size())
			{
				m_current_info++;
				m_current_round = m_player_info[m_current_info]->round;
				m_current_info_string = 0;
			}
			else
				m_current_info_string++;
			if (m_current_info_string < m_player_info[m_current_info]->strings.size())
				m_current_message = m_player_info[m_current_info]->strings[m_current_info_string];
		}
	}
}
void	Ui::prev_step()
{
	m_current_message = "";
	if (m_current_round > m_player_info[m_current_info]->round && m_current_info+1 == m_player_info.size())
	{
		m_current_round--;
		if (m_current_round == m_player_info[m_current_info]->round)
		{
			m_current_info_string = MAX(0,m_player_info[m_current_info]->strings.size()-1);
			if (m_current_info_string < m_player_info[m_current_info]->strings.size())
				m_current_message = m_player_info[m_current_info]->strings[m_current_info_string];
		}
	}
	else
	{
		if (m_current_info_string > 0)
			m_current_info_string--;
		else
		{
			if (m_current_info > 0)
			{
				m_current_info--;
				m_current_round = m_player_info[m_current_info]->round;
				m_current_info_string = MAX(0,(int)(m_player_info[m_current_info]->strings.size())-1);
			}
		}
		if (m_current_info_string < m_player_info[m_current_info]->strings.size())
			m_current_message = m_player_info[m_current_info]->strings[m_current_info_string];
	}
}
